#!/usr/bin/env python
# -*- coding: utf_8 -*-

import os
import re
import copy
import time
import types
import json
import logging
import traceback
import shutil
from collections import OrderedDict
from urllib.parse import urlparse, urljoin
from datetime import datetime
from pony.orm import *
from flask import request
from app.setting import config, conf
from model import fullStackDB
from model.download import AppDownload
from model.framework import Framework
from model.apps import Apps
from model.user import User
from utils import sql_filter
from utils.tools_epk import EpkApp

logger = logging.getLogger("DownloadManager")

def convert_url_to_local_path(url):
    parsed_result = urlparse(url)

    target_file = os.sep.join([config.get("UPLOAD_PATH").replace('\\', '/'), parsed_result.path])
    target_file = os.path.normpath(target_file)

    return target_file

class DownloadManager(object):
    def __init__(self):
        super(DownloadManager, self).__init__()

    def add(self, data):
        with db_session:
            app = Apps.get(id=data.get("app"), is_delete=False)
            if not app:
                return False, "app does not found"

            editor = User.get(id=request.current_user.get("id"))
            if not editor:
                return False, "current user is not exists"

            data.update({
                'create_by': editor,
                'create_at': datetime.now(),
                'update_by': editor,
                'update_at': datetime.now(),
            })

            result = AppDownload(app=app, imei=data.get("imei"), **data)
            commit()
            return result, "add dictionary {}.".format("success" if result else "fail")

    def delete(self, uuid):
        with db_session:
            editor = User.get(id=request.current_user.get("id"))
            if not editor:
                return False, "current user is not exists"

            result = AppDownload.get(uuid=uuid)
            if not result:
                return False, "dict does not exists"

            result.set(is_delete=True, delete_at=datetime.now(), delete_by=editor)
            commit()

            return result, "delete dictionary {}.".format("success" if result else "fail")

    def get(self, data):
        # 流程如下：
        # 获取前端传来的应用UUID和IMEI
        # 根据应用UUID查找应用信息，找不到返回失败信息
        # 如果应用ID是evue_launcher，则进行如下处理
        #   遍历系统所有应用，将相关应用根据4个一组拼接到一个数组里面去
        #   最后将这些信息导出为一个JSON文件
        #   将evue_launcher.evue和evue_dock.evue以及相关资源文件进行打包
        #   以上文件全部是复制处理，最后根据格式生成文件夹，将epk文件写入到这里
        #   读取这个epk文件，以encoding='utf8'格式返回字节流
        # 否则就是普通应用
        #   查找出这个应用路径以及依赖文件
        #   将这些文件进行打包
        #   读取这个epk文件，以encoding='utf8'格式返回字节流
        # 此次下载将生成一次下载记录

        # 当前还没有校验前端传来的IMEI是否是合法的
        epk_path = None
        with db_session:
            # 判断应用UUID是否等于evue_launcher
            if data.get("id") == "evue_launcher":
                # 读取evue_launcher.evue和evue_dock.evue以及相关资源文件
                # launcher_file = Apps.get(name="evue_launcher")
                # if not launcher_file:
                #     launcher_file = Framework.get(name="evue_launcher")
                #     if not launcher_file:
                #         return False, "evue_launcher not found"
                # else:
                #     launcher_file = launcher_file.annex

                # dock_file = Framework.get(name="evue_dock")
                # if not dock_file:
                #     return False, "evue_launcher not found"

                # 按照格式创建文件夹
                dir_format = datetime.now().strftime("%Y%m%d%H%M%S")
                build_path = os.sep.join([os.getcwd(), config.get("UPLOAD_DIR"), conf.get('application', 'launcher_dir'), data.get("imei"), dir_format])
                if not os.path.exists(build_path):
                    os.makedirs(build_path)

                build_source_path = os.sep.join([build_path, "src"])
                if not os.path.exists(build_source_path):
                    os.mkdir(build_source_path)

                # target_path = launcher_file.assets.get("files", []) + dock_file.assets.get("files", [])
                # # 将取evue_launcher.evue和evue_dock.evue以及相关资源文件复制到这个目录下
                # for f in target_path:
                #     tf = convert_url_to_local_path(f)
                #     if not os.path.exists(tf):
                #         return False, "{} not found".format(tf)
                    
                #     filename = os.path.basename(tf)
                #     name, suffix = os.path.splitext(filename)
                #     name = re.sub(r"_\d{14}$", "", name)
                #     shutil.copy(tf, os.sep.join([build_source_path, name + suffix]))

                temp = []
                # 读取当前系统所有应用及其资源文件
                apps = Apps.select().where(is_delete=False).order_by(Apps.sort)
                result_json = []
                for val in apps:
                    tmp = val.to_dict(with_collections=True, related_objects=True)
                    if val.app_build_log:
                        temp.append({
                            'icon': val.app_icon.title,
                            'name': val.app_name,
                            'url': val.app_url,
                            'version': val.app_version,
                            'epk': tmp.get("app_build_log")[0].app_path,
                            'category': val.category,
                            'id': str(val.uuid),
                        })

                    if val.app_annex:
                        # 需要将每个应用的文件拷贝到目标文件夹中
                        for f in val.app_annex:
                            tf = convert_url_to_local_path(f.path)
                            if not os.path.exists(tf):
                                return False, "{} not found".format(tf)
                            
                            filename = os.path.basename(tf)
                            name, suffix = os.path.splitext(filename)
                            name = re.sub(r"_\d{14}$", "", name)
                            shutil.copy(tf, os.sep.join([build_source_path, name + suffix]))

                    if len(temp) % 4 == 0 and temp:
                        result_json.append(copy.deepcopy(temp))
                        temp = []

                if temp:
                    result_json.append(copy.deepcopy(temp))
                json_str = json.dumps(result_json, sort_keys=False, ensure_ascii=False)
                with open(os.sep.join([build_source_path, 'evue_dock_apps.json']), 'w') as json_file:
                    json_file.write(json_str)

                # 打包成EPK文件
                epk = EpkApp(appName="evue_launcher", appDir=build_source_path, appVersion="1.0", output=build_path)
                epk.pack()
                epk_path = os.sep.join([build_path, 'evue_launcher.epk'])
            else:
                # 根据应用UUID查找相关应用
                app = Apps.get(uuid=data.get("id"), is_delete=False)
                if not app:
                    return False, "app not found"

                if not app.app_build_log:
                    return False, "app build file not found"

                tmp = app.to_dict(with_collections=True, related_objects=True)
                epk_path = convert_url_to_local_path(tmp.get("app_build_log")[0].app_path)
                if not os.path.exists(epk_path):
                    return False, "epk file not found"

                # 创建一条下载记录
                if app:
                    AppDownload(app=app, imei=data.get("imei"))
                    commit()

            return epk_path, "get dictionary {}.".format("success" if epk_path else "no data")

    def getList(self, data):
        if not data or len(data) <= 0:
            return False, 0, "parameters can not be null."

        temp = copy.deepcopy(data)
        if 'pagenum' in temp:
            temp.pop('pagenum')
        if 'pagesize' in temp:
            temp.pop('pagesize')
        if 'scope_type' in temp:
            temp.pop('scope_type')
        temp.setdefault("is_delete", False)

        with db_session:
            if "scope_type" in data and data.get("scope_type") == "list":
                result = AppDownload.select().where(**temp).order_by(AppDownload.sort)
                temp = []
                for item in result:
                    temp.append(item.to_dict(only=["uuid"]))
                return temp, len(temp), "get dictionary list {}.".format("success" if temp else "no data")

            result = AppDownload.select().where(**temp).order_by(desc(AppDownload.download_at)).page(data.get("pagenum", 1), data.get("pagesize", 10))
            count = AppDownload.select().where(**temp).count()

            if result:
                temp = []
                for item in result:
                    t = item.to_dict(with_collections=True, related_objects=True)
                    t.update({
                        "app": item.app.to_dict(exclude=["create_by", "update_by", "delete_by", "is_delete"]),
                        "download_at": item.download_at.strftime("%Y-%m-%d %H:%M:%S")
                    })
                    temp.append(t)
                result = temp

            return result, count, "get dictionary {}.".format("success" if result else "fail")

    def update(self, uuid, data):
        # 当参数为空时，直接返回错误
        if len(data) <= 0 or (len(data.keys()) == 1 and "id" in data):
            return False, "parameters can not be null."

        with db_session:
            # 查询请求者是否存在
            editor = User.get(id=request.current_user.get("id"))
            if not editor:
                return False, "current user is not exists"

            result = AppDownload.get(uuid=uuid)
            if not result:
                return False, "app does not found"

            result.set(update_at=datetime.now(), update_by=editor, **data)
            commit()

            return result, "update dictionary {}.".format("success" if result else "fail")

downloadManager = DownloadManager()
